@@ -21,7 +21,7 @@ class GenomicAnnotation:
     @staticmethod
     def select_anno(
         annoset: AbstractSet[str],
-        priority: Tuple[str, ...] = ("Promoter", "5UTR", "3UTR", "CDS", "Exon", "Intron", "Downstream", "Intergenic")
+        priority: Tuple[str, ...] = ("5UTR", "3UTR", "CDS", "Exon", "Intron", "Promoter", "Downstream", "Intergenic")
     ):
         for anno in priority:
             if anno in annoset:
@@ -29,7 +29,7 @@ class GenomicAnnotation:
 
     def primary_anno(
         self,
-        priority: Tuple[str, ...] = ("Promoter", "5UTR", "3UTR", "CDS", "Exon", "Intron", "Downstream", "Intergenic")
+        priority: Tuple[str, ...] = ("5UTR", "3UTR", "CDS", "Exon", "Intron", "Promoter", "Downstream", "Intergenic")
     ):
         return self.select_anno(self.detail, priority)
 
@@ -45,19 +45,71 @@ class AnnoSet:
     anno: List[str] = field(init=False, default_factory=list)
 
     def __post_init__(self):
+        first = []
+        second = []
+        third = []
+        fourth = []
+        other = []
         for anno in self.annoset:
-            self.id.append(anno.id)
-            self.name.append(anno.name)
-            self.start.append(anno.start)
-            self.end.append(anno.end)
-            self.type.append(anno.type)
-            self.anno.append(anno.primary_anno())
+            if not {"Promoter", "Downstream", "Intergenic"} & anno.detail:
+                if not anno.primary_anno() == "Intron":
+                    first.append(anno)
+                else:
+                    third.append(anno)
+            else:
+                if {"5UTR", "3UTR", "CDS", "Exon"} & anno.detail:
+                    second.append(anno)
+                elif "Intron" in anno.detail:
+                    fourth.append(anno)
+                else:
+                    other.append(anno)
+        if first:
+            for anno in first:
+                self.id.append(anno.id)
+                self.name.append(anno.name)
+                self.start.append(anno.start)
+                self.end.append(anno.end)
+                self.type.append(anno.type)
+                self.anno.append(anno.primary_anno())
+        elif second:
+            for anno in second:
+                self.id.append(anno.id)
+                self.name.append(anno.name)
+                self.start.append(anno.start)
+                self.end.append(anno.end)
+                self.type.append(anno.type)
+                self.anno.append(anno.primary_anno())
+        elif third:
+            for anno in third:
+                self.id.append(anno.id)
+                self.name.append(anno.name)
+                self.start.append(anno.start)
+                self.end.append(anno.end)
+                self.type.append(anno.type)
+                self.anno.append(anno.primary_anno())
+        elif fourth:
+            for anno in fourth:
+                self.id.append(anno.id)
+                self.name.append(anno.name)
+                self.start.append(anno.start)
+                self.end.append(anno.end)
+                self.type.append(anno.type)
+                self.anno.append(anno.primary_anno())
+        else:
+            for anno in other:
+                self.id.append(anno.id)
+                self.name.append(anno.name)
+                self.start.append(anno.start)
+                self.end.append(anno.end)
+                self.type.append(anno.type)
+                self.anno.append(anno.primary_anno())
 
     def primary_anno(
         self,
-        priority: Tuple[str, ...] = ("Promoter", "5UTR", "3UTR", "CDS", "Exon", "Intron", "Downstream", "Intergenic")
+        priority: Tuple[str, ...] = ("5UTR", "3UTR", "CDS", "Exon", "Intron", "Promoter", "Downstream", "Intergenic")
     ) -> str:
-        return GenomicAnnotation.select_anno(set(self.anno), priority)
+        anno = GenomicAnnotation.select_anno(set(self.anno), priority)
+        return anno
 
     def __str__(self) -> str:
         _id = ",".join(self.id)
@@ -109,13 +161,15 @@ class GFeature(ABC):
     def utr3_len(self) -> int:
         return blocks_len(self.utr3_exons())
 
-    def anno(self, blocks: List[Tuple[int, int]], region: Tuple[int, int] = (-1000, 1000), down: int = 3000) -> Set[str]:
+    def anno(self, blocks: List[Tuple[int, int]], region: Tuple[int, int] = (-3000, 0), down: int = 3000) -> Set[str]:
         anno = []
-        tss = intervals_is_overlap(blocks, [self.tss_region(region=region)])
-        introns = self.introns()
-        if introns:
-            introns = intervals_is_overlap(blocks, introns)
-        downstream = intervals_is_overlap(blocks, [self.downstream(down=down)])
+        st = self.tss_region(region=region)
+        end = self.downstream(down=down)
+        pos = sorted([*st, *end])
+        if blocks[0][0] < pos[0] or blocks[-1][1] > pos[3]:
+            anno.append("Intergenic")
+        tss = intervals_is_overlap(blocks, [st])
+        downstream = intervals_is_overlap(blocks, [end])
         if self.is_protein_coding():
             utr5_exons = self.utr5_exons()
             if utr5_exons:
@@ -144,10 +198,12 @@ class GFeature(ABC):
                 anno.append("CDS")
         elif exons:
             anno.append("Exon")
-        if introns:
-            anno.append("Intron")
         if downstream:
             anno.append("Downstream")
         if not anno:
-            anno.append("Intergenic")
+            anno.append("Intron")
+        elif not {"5UTR", "3UTR", "CDS", "Exon"} & set(anno):
+            if intervals_is_overlap(blocks, [(pos[1], pos[2])]):
+                anno.append("Intron")
+
         return set(anno)
