@@ -5,6 +5,7 @@ from io import TextIOWrapper
 from typing import List, Sequence, Tuple, Literal, Iterable, Iterator, Optional, Union, TextIO
 
 from pybiotk.annodb import Transcript
+from pybiotk.utils import logging
 from pybiotk.io.bed import Bed6, Bed12, Intron, GeneInfo, TransInfo
 from stream.pipe import Pipe, sort, drop_while, filter, kgroupby, groupby, apply, window
 
@@ -32,34 +33,58 @@ class GTF:
 
     @staticmethod
     def parse_attributes(term: str, attributes: str) -> Optional[str]:
-        parse_attr = re.compile('{term} "([^"]+)"'.format(term=term))
+        parse_attr = re.compile(f'{term} "([^"]+)"')
         patterns = parse_attr.findall(attributes)
 
         if patterns:
             return patterns[0]
-        else:
-            return None
 
     def gene_type(self):
-        return self.parse_attributes("gene_type", self.attributes)
+        for term in ['gene_type', 'gene_biotype']:
+            attr = self.parse_attributes(term, self.attributes)
+            if attr is not None:
+                return attr
 
     def gene_id(self):
         return self.parse_attributes("gene_id", self.attributes)
 
     def gene_name(self):
-        return self.parse_attributes("gene_name", self.attributes)
+        for term in ["gene_name", "gene"]:
+            attr = self.parse_attributes(term, self.attributes)
+            if attr is not None:
+                return attr
 
     def transcript_type(self):
-        return self.parse_attributes("transcript_type", self.attributes)
+        for term in ["transcript_type", "transcript_biotype"]:
+            attr = self.parse_attributes(term, self.attributes)
+            if attr is not None:
+                return attr
 
     def transcript_id(self):
-        return self.parse_attributes("transcript_id", self.attributes)
+        attr = self.parse_attributes("transcript_id", self.attributes)
+        if attr is None:
+            attr = self.gene_id()
+        return attr
 
     def transcript_name(self):
-        return self.parse_attributes("transcript_name", self.attributes)
+        attr = self.parse_attributes("transcript_name", self.attributes)
+        if attr is None:
+            attr = self.gene_name()
+        return attr
 
     def get_attribute(self, attribute):
-        return self.parse_attributes(attribute, self.attributes)
+        if attribute == "gene_type":
+            return self.gene_type()
+        elif attribute == "gene_name":
+            return self.gene_name()
+        elif attribute == "transcript_id":
+            return self.transcript_id()
+        elif attribute == "transcript_type":
+            return self.transcript_type()
+        elif attribute == "transcript_name":
+            return self.transcript_name()
+        else:
+            return self.parse_attributes(attribute, self.attributes)
 
 
 @Pipe
@@ -76,28 +101,39 @@ def to_Bed6(iterable: Iterable[Tuple[str, Tuple[GTF, ...]]]) -> Iterator[Bed6]:
 
 
 @Pipe
-def to_GeneInfo(iterable: Iterable[Tuple[str, Tuple[GTF, ...]]]) -> Iterator[GeneInfo]:
-    for (k, gtfs) in iterable:
+def to_GeneInfo(iterable: Iterable[Tuple[GTF, ...]]) -> Iterator[GeneInfo]:
+    for gtfs in iterable:
         for gtf in gtfs:
-            yield GeneInfo.init_by_gtf(gtf, k)
+            yield GeneInfo.init_by_gtf(gtf)
 
 
 @Pipe
-def to_TransInfo(iterable: Iterable[Tuple[str, Tuple[GTF, ...]]]) -> Iterator[TransInfo]:
-    for (k, gtfs) in iterable:
-        for gtf in gtfs:
-            yield TransInfo.init_by_gtf(gtf, k)
+def to_TransInfo(iterable: Iterable[Tuple[GTF, ...]]) -> Iterator[TransInfo]:
+    for group in iterable:
+        gene_gtf: Sequence[GTF] = []
+        trans_gtf: Sequence[GTF] = []
+        for gtf in group:
+            if gtf.feature == "gene":
+                gene_gtf.append(gtf)
+            else:
+                trans_gtf.append(gtf)
+        for gtf in trans_gtf:
+            trans_info = TransInfo.init_by_gtf(gtf)
+            if gene_gtf :
+                if trans_info.gene_type is None:
+                    trans_info.gene_type = gene_gtf[0].gene_type()
+            yield trans_info
 
 
 @Pipe
 def to_Bed12(iterable: Iterable[Tuple[GTF, ...]], name: str = "transcript_id") -> Iterator[Bed12]:
-
     for group in iterable:
         gtfs: Iterator[GTF] = group | sort(key=lambda x: x.start)
-
         cds_exons: Sequence[GTF] = []
         bed: Optional[Bed12] = None
+        last_gtf = None
         for gtf in gtfs:
+            last_gtf = gtf
             if gtf.feature in {'CDS', 'stop_codon'}:
                 cds_exons.append(gtf)
             elif gtf.feature == "exon":
@@ -105,12 +141,14 @@ def to_Bed12(iterable: Iterable[Tuple[GTF, ...]], name: str = "transcript_id") -
                     bed = Bed12.init_by_gtf(gtf, gtf.get_attribute(name))
                 else:
                     bed.update(gtf)
+        if bed is None:
+            logging.warning(f"No exon found for transcript: {last_gtf.get_attribute(name)}")
+            continue
         if cds_exons:
             bed.thickStart = cds_exons[0].start - 1
             bed.thickEnd = cds_exons[-1].end
         else:
             bed.thickStart = bed.thickEnd = bed.end
-
         yield bed
 
 
@@ -127,27 +165,44 @@ def to_Intron(iterable: Iterable[Tuple[GTF, ...]], name: str = "transcript_id")
 @Pipe
 def to_Transcript(iterable: Iterable[Tuple[GTF, ...]]) -> Iterator[Transcript]:
     for group in iterable:
-        gtfs: Iterator[GTF] = group | sort(key=lambda x: x.start)
+        gene_gtf: Sequence[GTF] = []
+        exons_gtf: Sequence[GTF] = []
+        for gtf in group:
+            if gtf.feature == "gene":
+                gene_gtf.append(gtf)
+            else:
+                exons_gtf.append(gtf)
 
-        cds_exons: Sequence[GTF] = []
-        transcript: Optional[Transcript] = None
-        for gtf in gtfs:
-            if gtf.feature in {'CDS', 'stop_codon'}:
-                cds_exons.append(gtf)
-            elif gtf.feature == "exon":
-                if transcript is None:
-                    transcript = Transcript.init_by_gtf(gtf)
-                else:
-                    transcript.update(gtf)
-        if cds_exons:
-            cds_start = cds_exons[0].start - 1
-            cds_end = cds_exons[-1].end
-            transcript.cds_start = cds_start
-            transcript.cds_end = cds_end
-        else:
-            transcript.cds_start = transcript.cds_end = None
+        for group in exons_gtf | groupby(key=lambda x: x.transcript_id()):
+            gtfs: Iterator[GTF] = group | sort(key=lambda x: x.start)
+            cds_exons: Sequence[GTF] = []
+            transcript: Optional[Transcript] = None
+            for gtf in gtfs:
+                if gtf.feature in {'CDS', 'stop_codon'}:
+                    cds_exons.append(gtf)
+                elif gtf.feature == "exon":
+                    if transcript is None:
+                        transcript = Transcript.init_by_gtf(gtf)
+                    else:
+                        transcript.update(gtf)
+            if transcript is None:
+                logging.warning(f"No exon found for transcript: {gtf.get_attribute('transcript_id')}")
+                continue
+            if cds_exons:
+                cds_start = cds_exons[0].start - 1
+                cds_end = cds_exons[-1].end
+                transcript.cds_start = cds_start
+                transcript.cds_end = cds_end
+            else:
+                transcript.cds_start = transcript.cds_end = None
+    
+            if gene_gtf:
+                if transcript.gene_type is None:
+                    transcript.gene_type = gene_gtf[0].gene_type()
+            if transcript.transcript_type is None:
+                transcript.transcript_type = transcript.gene_type
 
-        yield transcript
+            yield transcript
 
 
 class GtfFile:
@@ -323,7 +378,7 @@ class GtfFile:
             transcript_names=transcript_names,
             gene_ids=gene_ids,
             gene_names=gene_names
-            ) | kgroupby(lambda x: x.gene_id()) | to_GeneInfo
+            ) | groupby(lambda x: x.gene_id()) | to_GeneInfo
         return gene_info
 
     def to_trans_info(self,
@@ -334,14 +389,15 @@ class GtfFile:
                       gene_ids: Optional[Sequence[str]] = None,
                       gene_names: Optional[Sequence[str]] = None,
                       ) -> Iterator[TransInfo]:
-        trans_info = self.iter_transcript(
+        trans_info = self.iter(
+            features=["gene", "transcript"],
             gene_types=gene_types,
             transcript_types=transcript_types,
             transcript_ids=transcript_ids,
             transcript_names=transcript_names,
             gene_ids=gene_ids,
             gene_names=gene_names
-            ) | kgroupby(lambda x: x.transcript_id()) | to_TransInfo
+            ) | groupby(lambda x: x.gene_id()) | to_TransInfo
         return trans_info
 
     def to_bed12(self, name: Literal["gene_id", "gene_name", "transcript_id", "transcript_name"] = "transcript_id",
@@ -388,12 +444,12 @@ class GtfFile:
                       gene_ids: Optional[Sequence[str]] = None,
                       gene_names: Optional[Sequence[str]] = None,
                       ) -> Iterator[Transcript]:
-        transcript = self.iter(features=["CDS", 'stop_codon', 'exon'],
+        transcript = self.iter(features=["gene", "CDS", 'stop_codon', 'exon'],
                                gene_types=gene_types,
                                transcript_types=transcript_types,
                                transcript_ids=transcript_ids,
                                transcript_names=transcript_names,
                                gene_ids=gene_ids,
                                gene_names=gene_names
-                               ) | groupby(lambda x: x.transcript_id()) | to_Transcript
+                               ) | groupby(lambda x: x.gene_id()) | to_Transcript
         return transcript
