@@ -55,10 +55,10 @@ def df_merge_row(df, by=None, columns=None, method=("distinct",), sep=",", count
     return merge_df
 
 
-def main(filename, output, header, noheader, by, columns=None, method="distinct", sep=None, delim=",", count=False):
+def main(input, output, header, noheader, by, columns=None, method="distinct", sep=None, delim=",", count=False):
     if noheader or header < 0:
         header = None
-    df = read_table(filename, header=header, sep=sep)
+    df = read_table(input, header=header, sep=sep)
     out_header = True if header is not None else False
     if df.empty:
         write_table(df, output, header=out_header)
@@ -101,8 +101,8 @@ def run():
     parser = argparse.ArgumentParser(
         description=__doc__,
         formatter_class=argparse.ArgumentDefaultsHelpFormatter)
-    parser.add_argument("-i", "--input", dest="input", type=argparse.FileType('r'),
-                        default=(None if sys.stdin.isatty() else sys.stdin), help="Input table file.")
+    parser.add_argument("-i", "--input", dest="input", type=str,
+                        default=(None if sys.stdin.isatty() else "-"), help="Input table file.")
     parser.add_argument("-o", "--output", dest="output", type=argparse.FileType('w'),
                         default=sys.stdout, help="Output file, discard this parameter to print output to stdout.")
     parser.add_argument("--header", dest="header", type=int, default=0,
@@ -129,9 +129,10 @@ def run():
     args = parser.parse_args()
 
     if args.input is None:
-        parser.parse_args(['-h'])
+        parser.print_help()
+        sys.exit(1)
 
-    main(**args.__dict__)
+    main(**vars(args))
 
 
 if __name__ == "__main__":
